import * as MobX from "mobx";
import { ComponentClass } from "react";

/**
 * @see {@link https://github.com/mobxjs/mobx/blob/main/packages/mobx-react/README.md#note-on-using-props-and-state-in-derivations}
 */
export function observePropsState<T extends ComponentClass>(
  ComponentBaseClass: T
) {
  class ObservedComponent extends (ComponentBaseClass as ComponentClass) {
    constructor(props: InstanceType<ComponentClass>["props"]) {
      super(props);
      MobX.makeObservable?.(this);
    }

    @MobX.observable
    observedProps = this.props;

    @MobX.observable
    observedState = {} as InstanceType<ComponentClass>["state"];

    protected syncPropsState() {
      this.observedProps = this.props;
      this.observedState = this.state;
    }

    componentDidMount() {
      this.syncPropsState();

      super.componentDidMount?.();
    }

    componentDidUpdate(
      prevProps: Readonly<InstanceType<ComponentClass>["props"]>,
      prevState: Readonly<InstanceType<ComponentClass>["state"]>,
      snapshot?: any
    ) {
      this.syncPropsState();

      super.componentDidUpdate?.(prevProps, prevState, snapshot);
    }
  }
  return ObservedComponent as unknown as T;
}
